package org.eso.ohs.core.docview.datatrans;

/**
 *  A transducer to convert set/get of String values into set/get
 *  on a boolean property.  The target property's name is given to the
 *  constructor.
 *
 *  <p>
 *  For example, users will call it's
 *  setValue/getValue methods.  Those calls will be translated into
 *  calls to setX/getX on the object given to the constructor.
 *
 *  <p>
 * $Id: BooleanAdaptor.java,v 1.1 2004/02/17 12:37:59 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 */
public class BooleanAdaptor extends ObjAdaptor {

    /**
     * Construct a transducer.
     */
    public BooleanAdaptor() {
	super(boolean.class);
    }

    /**
     * Construct a transducer to convert set/get of String values into
     * setX/getX on an int property of <code>object</code>.
     */
    public BooleanAdaptor(Object obj, String fieldName) {
	super(boolean.class, obj, fieldName);
    }

    /**
     * Converts data from a String to a meta-level equivalent of the
     * actual property's type, i.e. to a java.lang.Boolean
     * representing a boolean.  Used with reflection to set
     * the new property value.
     *
     * @param value	possible value of the property, as a String.
     *
     * @return		same value of the property as a wrapper type,
     *			eg a boolean wrapped in a java.lang.Boolean.
     */
    public Object toMetaLevelObject(String value) {
	return new Boolean(value);
    }

    /**
     * Converts data from a String to a meta-level equivalent of the
     * actual property's type, i.e. to a java.lang.Boolean
     * representing a boolean.  Used with reflection to set
     * the new property value.
     *
     * @param value	possible value of the property as a wrapper type.
     *
     * @return		same value of the property as a String.
     */
    public String fromMetaLevelObject(Object obj) {
	return obj == null ? "false" : obj.toString();
    }

}

