package org.eso.ohs.core.docview.datatrans;

/**
 * An interface to insulate DocView from the adaptor implementations.
 * 
 * <p>
 * $Id: ObjectAdaptor.java,v 1.1 2004/02/17 12:37:59 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 */
public interface ObjectAdaptor extends Adaptor {

    /**
     * Get the raw (unconverted) value of the objects property
     * 
     * @return the current raw value of the property.
     */
    public Object getRawValue();

    /**
     * Returns the current Object.
     * 
     * @return			current Object
     */
    public Object getObject();

    /**
     * Sets the current Object.
     * 
     * @param obj	new Object
     */
    public void setObject(Object obj);

    /**
     * Returns the current property name.
     * 
     * @return			property name
     */
    public String getProperty();

    /**
     * Sets the current property name.
     * 
     * @param name	property name
     */
    public void setProperty(String name);

    /**
     * Returns true if the current property is modifiable
     * 
     * @return			property name
     */
    public boolean getReadWrite();

    /**
     * Enables or disables modification of the value of the object's property.
     * 
     * @param readWrite		true if it can be updated, false otherwise
     */
    public void setReadWrite(boolean readWrite);

    /**
     * Converts data to a String from the meta-level equivalent of the
     * actual property's type, eg from a java.lang.Long
     * to a long int.  Used with reflection to get
     * the current property value.
     *
     * @param value	value of the property.
     */
    public String fromMetaLevelObject(Object obj);

    /**
     * Converts data from a String to a meta-level equivalent of the
     * actual property's type, eg to a java.lang.Long
     * representing a long int.  Used with reflection to set
     * the new property value.
     *
     * @param value the new value of the property.
     */
    public Object toMetaLevelObject(String obj);

}

