package org.eso.ohs.core.gui.models;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;


/**
 * A Document model to only permit the input of a Duration.
 * in the format of 31:21:30:00
 * <p>
 * $Id: TimeDurationModel.java,v 1.1 2004/02/17 10:35:21 tcanavan Exp $
 */

public class TimeDurationModel extends InputMaskModel {

    private Character xCol = new Character(':');
    private Character xT = new Character('T');


    /**
     * Legal values if they are valid
     * Looking for values in the following 
     * format "d'T'HH:mm:ss"
     *
     */
    private final static String legalValuesPattern_ =
        "^(\\d{0,2}(T\\d{0,2}(:\\d{0,2}(:\\d{0,2})?)?)?)?";

    /**
     * Constructor for the model.
     */
    public TimeDurationModel() {
	super(legalValuesPattern_);
    }

    /**
        * Refuse input that is above or below the limits of the
        * textfield.
        */
    public void insertString(int offset, String str, AttributeSet a)
        throws BadLocationException {

        //stdlog_.debug("String: <"+str+">");
        super.insertString(offset, str, a);
        //stdlog_.debug("buffer: <"+buffer_+">");
        try {
            String buffer_val = buffer_.toString();
            int len = buffer_val.length();
            //stdlog_.debug("len: <"+len+">");
            if (len > 1 ){
                /** Check for double  : or a : after the T*/
                Character x = new Character(buffer_val.charAt(len-2));
                if (x.equals(xCol) && str.equals(":"))
                    remove(offset, str.length());
                else if (x.equals(xT) && str.equals(":"))
                    remove(offset, str.length());
            }
        } catch (NumberFormatException e) {
            // nothing, the string is not complete...
        }
    }

}

