package org.eso.ohs.core.utilities.rmi;

import java.io.IOException;
import java.net.ServerSocket;

/**
 * Socket factory class to extend the standard RMI socket factory:
 * allow server objects to listen to a specific port.
 *
 * Modifications for P2PP Server release 2.4:
 *
 * This class now extends RMIMasterSocketFactory. This allows for both
 * the remapping of an anonymous port as well as the creation of a socket
 * for handling both RMI and HTTP requests.
 *
 * @version $Revision: 1.2 $
 *
 * @author amchavan, 11-Oct-2001: factored out from AppServer.java and
 *                                DbbServer.java
 */
public class OHSRMISocketFactory extends
       sun.rmi.transport.proxy.RMIMasterSocketFactory {
    
    private static final String rcsid = "$Id: OHSRMISocketFactory.java,v 1.2 2005/05/04 13:26:49 gdonino Exp $";

    /**
     * The default port number for 'anonymous' server sockets. 
     * @see java.rmi.server.UnicastRemoteObject
     */
    public static final int DEFAULT_ANONYMOUS_PORT = 1199;

    /**
     * The port port number for 'anonymous' server sockets. 
     */
    private int anonymousPort_;

    /** Default constructor */
    public OHSRMISocketFactory() {
	anonymousPort_ = DEFAULT_ANONYMOUS_PORT;
    }

    /** 
     * Constructor: defines the port number for 'anonymous' server
     * sockets.
     *
     * @param anonymport  Anonymous port number.
     *
     * @see #DEFAULT_ANONYMOUS_PORT
     * @see java.rmi.server.UnicastRemoteObject
     *
     * <br>ChangeLog: amchavan, 11-Oct-2001: creation
     */

    public OHSRMISocketFactory( int anonymport ) {
	anonymousPort_ = anonymport;
    }

    
    /* (non-Javadoc)
     * @see java.rmi.server.RMIServerSocketFactory#createServerSocket(int)
     */
    public ServerSocket createServerSocket( int port ) throws IOException {
	if (port == 0) {
            port = anonymousPort_;
        }
	return super.createServerSocket(port);
    }
}
