package org.eso.ohs.dfs;

/**
 * 
 * 
 * <p>
 * $Id: ConstraintSetDTO.java,v 1.4 2004/12/21 19:15:22 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 */

public class ConstraintSetDTO {
    
    
    
    private int count;

    /** Maximum airmass: between 1 and 5 */
    private float airmass_ = 5.0f;

    /** Maximum Fractional Lunar Illumination: between 0 and 1 */
    private float fractionalLunarIllumination_ = 1.0f;

    /**
     * Minimum allowed angular distance between the target and the Moon,
     * in degress between 0 and 180
     */
    private int moonAngularDistance_ = 30;

    /**
     * Seeing or Image Quality: is specified to the R band.
     * Expressed as a number between 0.2 and 2.0 (image size in arcseconds)
     */
    private float seeing_ = 2.0f;

    /**
     * SkyTransparency can be defined as "Photometric", "Clear",
     * "Variable, thin cirrus", "Variable, thick cirrus"
     */
    private String skyTransparency_ = validSkyTransparency_[0];

    /**
     * Legal values for the SkyTransparency property
     */
    private static final String[] validSkyTransparency_ =
        {
            "Photometric",
            "Clear",
            "Variable, thin cirrus",
            "Variable, thick cirrus" };

    /**
     * Legal values for the SkyTransparency property in the database
     */
    private static final String[] skyTransparencyDbRange =
        { "1PHO", "2CLR", "3THN", "4THK" };



    /**
     * null constructor
     */
    public ConstraintSetDTO() {
    }

    /**
     * Returns an array of the valid values for the SkyTransparency property.
     * 
     * @return array of strings
     */
    public static String[] getValidSkyTransparency() {
        String[] valid = new String[validSkyTransparency_.length];
        System.arraycopy(validSkyTransparency_, 0, valid, 0, valid.length);

        return valid;
    }

    /**
     * Returns an array of the valid values for the SkyTransparency property.
     * 
     * @return array of strings
     */
    public static String[] getDbSkyTransparency() {
        String[] valid = new String[skyTransparencyDbRange.length];
        System.arraycopy(skyTransparencyDbRange, 0, valid, 0, valid.length);

        return valid;
    }

    /**
     * Returns the SkyTransparency value corresponding to a column value
     * in the database.
     * 
     * @return SkyTransparency value
     */
    public static String fromDbSkyTransparency(String dbValue) {
        for (int i = 0; i < skyTransparencyDbRange.length; i++) {
            if (skyTransparencyDbRange[i].equals(dbValue))
                return validSkyTransparency_[i];
        }
        return null;
    }

    /**
     * Returns the database column value corresponding to a
     * SkyTransparency value.
     * 
     * @return SkyTransparency value
     */
    public String toDbSkyTransparency(String viewValue) {
        for (int i = 0; i < validSkyTransparency_.length; i++) {
            if (validSkyTransparency_[i].equals(viewValue))
                return skyTransparencyDbRange[i];
        }
        return null;
    }

    /**
     * Get value for Airmass property
     *
     * @return the current value of the property
     */
    public float getAirmass() {
        return airmass_;
    }

    /**
     * Get value for FractionalLunarIllumination property
     *
     * @return the current value of the property
     */
    public float getFractionalLunarIllumination() {
        return fractionalLunarIllumination_;

    }

    /**
     * Get value for SkyTransparency property
     *
     * @return the current value of the property
     */
    public String getSkyTransparency() {
        return skyTransparency_;
    }

    /**
     * Get value for MoonAngularDistance property
     *
     * @return the current value of the property
     */
    public int getMoonAngularDistance() {
        return moonAngularDistance_;

    }

    /**
     * Get value for Seeing property
     *
     * @return the current value of the property
     */
    public float getSeeing() {
        return seeing_;
    }


    /**
     * Set new value for Airmass property
     *
     * @param newValue the new value for the property
     */
    public void setAirmass(float airmass) {
        if (airmass < 1 || airmass > 5) {
            throw new IllegalArgumentException("airmass not in range 1 to 5");
        }
        airmass_ = airmass;
    }


    /**
     * Set new value for FractionalLunarIllumination property
     *
     * @param newValue the new value for the property
     */
    public void setFractionalLunarIllumination(float fractIllum) {
        if (fractIllum < 0 || fractIllum > 1) {
            throw new IllegalArgumentException("fractional lunar illumination not in range 0 to 1");
        }

        fractionalLunarIllumination_ = fractIllum;
    }

    /**
     * Set new value for MoonAngularDistance property
     *
     * @param newValue the new value for the property
     */
    public void setMoonAngularDistance(int moonAngularDistance) {
        if (moonAngularDistance < 0 || moonAngularDistance > 180) {
            throw new IllegalArgumentException("moon angular distance not in range 0 to 180");
        }
        moonAngularDistance_ = moonAngularDistance;
    }

    /**
     * Set new value for Seeing property
     *
     * @param newValue the new value for the property
     */
    public void setSeeing(float seeing) {
        if (seeing < 0.2 || seeing > 2.0) {
            throw new IllegalArgumentException("seeing/image quality not in range 0.2 to 2.0");
        }
        seeing_ = seeing;
    }



    /**
     * Set new value for SkyTransparency property
     *
     * @param newValue the new value for the property
     */
    public void setSkyTransparency(String skyTransparency) {
        //for (int index = 0; index < slyTr_.length; index++) {
        //    if (validSkyTransparency_[index].equals(skyTransparency)) {
                skyTransparency_ = skyTransparency;
                //return;
            //}
        //}

        //throw new IllegalArgumentException(
        //    "sky transparency not recognised: " + skyTransparency);
    }
    /**
     * @return Returns the count.
     */
    public int getCount() {
        return count;
    }
    /**
     * @param count The count to set.
     */
    public void setCount(int count) {
        this.count = count;
    }
}
