package org.eso.ohs.dfs;

import org.eso.ohs.dfs.Seeing;
import org.eso.ohs.dfs.SkyTransparency;
import org.eso.ohs.dfs.StrehlRatio;


/**
 * Class WeatherConstraint
 *
 * contains the weather constraint 
 * to be used to transfer data from the view to the model in Orang
 * TODO Javadoc
 *
 * @version $Revision: 1.1 $
 * @author ddorigo, Nov 7, 2005
 */

// $Id: WeatherConstraint.java,v 1.1 2005/11/08 09:03:58 ddorigo Exp $
public class WeatherConstraint {

    private Seeing seeing;
    private SkyTransparency sky;
    private StrehlRatio strehlRatio;
    private boolean seeingEnable;
    private boolean skyEnable;
    private boolean strehlRatioEnable;

    /**
     * the orang weather constraints
     * @param seeing current seeing
     * @param sky current sky
     * @param strehlRatio current strehl ratio  (applicable to naco)
     * @param seeingEnable if the seeing data should be taken into account
     * @param skyEnable if the sky data should be taken into account
     * @param strehlRatioEnable  if the ratio data should be taken into account
     */
    public WeatherConstraint(Seeing seeing, SkyTransparency sky, StrehlRatio strehlRatio,
            boolean seeingEnable,
            boolean skyEnable, boolean strehlRatioEnable) {
     setSeeing(seeing);
     setSky(sky);
     setStrehlRatio(strehlRatio);
     setSeeingEnable(seeingEnable);
     setSkyEnable(skyEnable);
     setStrehlRatioEnable(strehlRatioEnable);
    } 
    
    /**
     * the orang weather constraints
     * @param seeing current seeing
     * @param sky current sky
     * @param strehlRatio current strehl ratio  (applicable to naco)
     */
    public WeatherConstraint(Seeing seeing, SkyTransparency sky, StrehlRatio strehlRatio) {
     this(seeing,sky,strehlRatio,true,true,true);
    }

    /**
     * @return Seeing
     */
    public Seeing getSeeing() {
        return seeing;
    }

    /**
     * @param seeing void
     */
    public void setSeeing(Seeing seeing) {
        this.seeing = seeing;
    }

    /**
     * @return boolean
     */
    public boolean isSeeingEnable() {
        return seeingEnable;
    }

    /**
     * @param seeingEnable void
     */
    public void setSeeingEnable(boolean seeingEnable) {
        this.seeingEnable = seeingEnable;
    }

    /**
     * @return SkyTransparency
     */
    public SkyTransparency getSky() {
        return sky;
    }

    /**
     * @param sky void
     */
    public void setSky(SkyTransparency sky) {
        this.sky = sky;
    }

    /**
     * @return boolean
     */
    public boolean isSkyEnable() {
        return skyEnable;
    }

    /**
     * @param skyEnable void
     */
    public void setSkyEnable(boolean skyEnable) {
        this.skyEnable = skyEnable;
    }

    /**
     * @return StrehlRatio
     */
    public StrehlRatio getStrehlRatio() {
        return strehlRatio;
    }

    /**
     * @param strehlRatio void
     */
    public void setStrehlRatio(StrehlRatio strehlRatio) {
        this.strehlRatio = strehlRatio;
    }

    /**
     * @return boolean the SkyTransparency can 
     */
    public boolean isStrehlRatioEnable() {
        return strehlRatioEnable;
    }

    /**
     * @param strehlRatioEnable void
     */
    public void setStrehlRatioEnable(boolean strehlRatioEnable) {
        this.strehlRatioEnable = strehlRatioEnable;
    }

}
