package org.eso.ohs.persistence.dbase.phase1;

import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.apache.log4j.Logger;
import org.eso.ohs.dfs.PanelObsRun;
import org.eso.ohs.dfs.StorableObject;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;
import org.eso.ohs.persistence.dbase.DbaseIOHandler;

/**
 * I/O class for PanelObsRuns stored in the database.
 *
 * <p>
 * $Id: DbaseHandlerPanelObsRun.java,v 1.1 2005/03/30 09:56:11 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 * 
 * @see org.eso.ohs.persistence.dbase.phase2.DbaseMgr
 *
 */

public class DbaseHandlerPanelObsRun extends DbaseIOHandler {

	/** Standard log channel */
	private static Logger stdlog_ =
		Logger.getLogger(DbaseHandlerPanelObsRun.class);

	public static final float GRADE_CONV = 10;
	public static final float TIME_CONV = 10;

	/**
	 * Constructor for CS I/O class.
	 */
	public DbaseHandlerPanelObsRun() {
	}

	/**
	 * Managed class
	 * @return PanelObsRun class
	 */

	public Class getManagedClass() {
		return PanelObsRun.class;
	}

	/**
	 * Reads the object from the database, partially reassembling it.
	 * References to other business objects are stored in other fields
	 * of the array
	 * 
	 * @param stmt  JDBC Statement for contacting the dbase
	 * @param id    per-table id of the object to be retrieved.
	 * @return	    array in the format required by reassembleReferences()
	 * 
	 * @exception SQLException
	 *		    an error occurred during the dbase operation
	 * @exception ObjectNotFoundException
	 *		    the requested object was not found in the database
	 * @exception IOException
	 *              if an I/O error occurred while reading the object
	 * 
	 * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
	 * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
	 */
	public UnpackedStorableObject read(Statement stmt, long id)
		throws ObjectNotFoundException, IOException, SQLException {
		ResultSet rs = null;
		PanelObsRun panel_run = (PanelObsRun) newInstance();
		UnpackedStorableObject uso = new UnpackedStorableObject(panel_run);

		String sql =
			"SELECT runid = "
				+ Phase1SelectStmt.programmesTable.get()
				+ ".proposalID+'('+"
				+ Phase1SelectStmt.runsTable.get()
				+ ".run_desc+')', "
				+ "d_grade = "
				+ Phase1SelectStmt.runsTable.get()
				+ ".grade/"
				+ GRADE_CONV
				+ ", int_grade = "
				+ Phase1SelectStmt.runsTable.get()
				+ ".grade, "
				+ "rank, "
				+ "d_assigned_time = "
				+ Phase1SelectStmt.runsTable.get()
				+ ".assigned_time/"
				+ TIME_CONV
				+ ", "
				+ "d_sigma = "
				+ Phase1SelectStmt.runsTable.get()
				+ ".sigma/"
				+ GRADE_CONV
				+ ", "
				+ Phase1SelectStmt.runsTable.get()
				+ ".obs_mode, "
				+ "dav_grade = av_grade/"
				+ GRADE_CONV
				+ ", "
				+ "dav_nights = av_nights/"
				+ TIME_CONV
				+ ", "
				+ Phase1SelectStmt.runsTable.get()
				+ ".tel_code, "
				+ Phase1SelectStmt.runsTable.get()
				+ ".inst_code, "
				+ Phase1SelectStmt.runsTable.get()
				+ ".requested_time, "
				+ Phase1SelectStmt.programmesTable.get()
				+ ".panel, "
				+ Phase1SelectStmt.programmesTable.get()
				+ ".opc_subpanel, "
				+ Phase1SelectStmt.programmesTable.get()
				+ ".period, "
				+ Phase1SelectStmt.runsTable.get()
				+ ".month, "
				+ Phase1SelectStmt.runsTable.get()
				+ ".moon, "
				+ Phase1SelectStmt.runsTable.get()
				+ ".seeing, "
				+ Phase1SelectStmt.runsTable.get()
				+ ".sky_transparency, "
				+ Phase1SelectStmt.programmesTable.get()
				+ ".title, "
				+ Phase1SelectStmt.runsTable.get()
				+ ".programme_id \n"
				+ "FROM "
				+ Phase1SelectStmt.runsTable.get()
				+ ", "
				+ Phase1SelectStmt.programmesTable.get()
				+ ", referee_avg\n"
				+ "WHERE obs_runs.id = "
				+ id
				+ "\n"
				+ "AND "
				+ Phase1SelectStmt.programmesTable.get()
				+ ".id = "
				+ Phase1SelectStmt.runsTable.get()
				+ ".programme_id\n"
				+ "AND "
				+ Phase1SelectStmt.runsTable.get()
				+ ".id = referee_avg.run_id\n";

		stdlog_.debug(sql);
		rs = stmt.executeQuery(sql);

		if (!rs.next()) { // Only one with this id !!
			throw new ObjectNotFoundException("Object not found in database");
		}

		panel_run.setId(Config.getCfg().tableToUniqueId(id, PanelObsRun.class));
		panel_run.setRunId(rs.getString("runid"));
		panel_run.setPeriod(rs.getInt("period"));
		panel_run.setGrade(rs.getFloat("d_grade"));
		panel_run.setRank(rs.getInt("rank"));
		panel_run.setNights(rs.getFloat("d_assigned_time"));
		panel_run.setSigma(rs.getFloat("d_sigma"));
		panel_run.setObsMode(rs.getString("obs_mode"));
		panel_run.setAvgGrade(rs.getFloat("dav_grade"));
		panel_run.setAvgNights(rs.getFloat("dav_nights"));
		panel_run.setTelescope(rs.getString("tel_code"));
		panel_run.setInstrument(rs.getString("inst_code"));
		panel_run.setReqTime(rs.getString("requested_time"));
		panel_run.setMonth(rs.getString("month"));
		panel_run.setMoon(rs.getString("moon"));
		panel_run.setSeeing(rs.getString("seeing"));
		panel_run.setSkyTransparency(rs.getString("sky_transparency"));
		panel_run.setTitle(rs.getString("title"));
		panel_run.setProgrammeDbId(rs.getLong("programme_id"));

		int int_grade = rs.getInt("int_grade");
		String panel = rs.getString("panel");
		int opc_subpanel = rs.getInt("opc_subpanel");
		int period = rs.getInt("period");

		rs.close();

		sql =
			"SELECT surname_dl FROM eso_users, proposed, obs_runs\n"
				+ "WHERE obs_runs.id = "
				+ id
				+ "\n"
				+ "AND obs_runs.programme_id = proposed.programme_id\n"
				+ "AND eso_users.id = proposed.user_id\n"
				+ "ORDER BY pi_flag\n";

		stdlog_.debug(sql);
		rs = stmt.executeQuery(sql);

		String applicants = "";
		while (rs.next())
			applicants += rs.getString("surname_dl") + " ";

		panel_run.setApplicants(applicants);
		rs.close();

		if (int_grade > 0 && panel_run.getRank() > 0) {
			sql =
				"SELECT max_rank = max(rank),\n"
					+ "min_rank = min(rank)\n"
					+ "FROM "
					+ Phase1SelectStmt.runsTable.get()
					+ ", "
					+ Phase1SelectStmt.programmesTable.get()
					+ "\n"
					+ "WHERE "
					+ Phase1SelectStmt.programmesTable.get()
					+ ".id = "
					+ Phase1SelectStmt.runsTable.get()
					+ ".programme_id\n"
					+ "AND "
					+ Phase1SelectStmt.runsTable.get()
					+ ".grade = "
					+ int_grade
					+ "\n"
					+ "AND "
					+ Phase1SelectStmt.programmesTable.get()
					+ ".panel = '"
					+ panel
					+ "'\n"
					+ "AND "
					+ Phase1SelectStmt.programmesTable.get()
					+ ".opc_subpanel = "
					+ opc_subpanel
					+ "\n"
					+ "AND "
					+ Phase1SelectStmt.runsTable.get()
					+ ".tel_code = '"
					+ panel_run.getTelescope()
					+ "'\n"
					+ "AND "
					+ Phase1SelectStmt.runsTable.get()
					+ ".rank > 0 \n"
					+ "AND "
					+ Phase1SelectStmt.programmesTable.get()
					+ ".period = "
					+ period
					+ "\n";

			stdlog_.debug(sql);
			rs = stmt.executeQuery(sql);
			while (rs.next()) {
				panel_run.setMaxRank(rs.getInt("max_rank"));
				panel_run.setMinRank(rs.getInt("min_rank"));
			}
			rs.close();
		}

		return uso;
	}

	/**
	 * Writes the object to the database, for the first time.
	 * 
	 * @param stmt  JDBC Statement for contacting the dbase
	 * @param id    per-table id of the object to be written.
	 * @param obj   an array: the first element is the BO to be stored.
	 * @exception SQLException  an error occurred during the dbase operation
	 * 
	 * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
	 * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
	 */
	public void write(Statement stmt, long id, UnpackedStorableObject obj)
		throws SQLException {
		// 	PanelObsRun panel_run = (PanelObsRun)obj.getObject();

		// 	StringBuffer sb = new StringBuffer();

		// 	sb.append(panel_run.getOBId());
		// 	sb.append(",");
		// 	sb.append(panel_run.getTSIndex());
		// 	sb.append(",");
		// 	sb.append(panel_run.getParamIndex());
		// 	sb.append(",");
		// 	sb.append(DbbSqlUtils.sqlEscapedString(panel_run.getPanelObsRunNAID()));
		// 	sb.append(",");
		// 	sb.append(panel_run.getPanelObsRunId());
		// 	sb.append(",");
		// 	sb.append(panel_run.getSlotNum());

		// 	String sqlText = "INSERT INTO masks"
		// 	    + " (grade, rank, assigned_time, sigma, obs_mode)"
		// 	    +   " mask_naid, mask_id, mask_slot_num)"
		// 	    + " VALUES (" + sb + ")";

		// 	stdlog_.debug(sqlText);
		// 	stmt.executeUpdate(sqlText);
	}
	/**
	  * Updates an object that is already stored in the database.
	  * 
	  * @param stmt  JDBC Statement for contacting the dbase
	  * @param id    per-table id of the object to be written.
	  * @param obj   an array: the first element is the BO to be stored.
	  * @exception SQLException  an error occurred during the dbase operation
	  * 
	  * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
	  * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
	  */
	public void update(Statement stmt, long id, UnpackedStorableObject objSpec)
		throws SQLException {
		stdlog_.debug("update object");
		StorableObject obj = objSpec.getObject();
		update(stmt, obj);
		stdlog_.debug("update object done");
	}

	/**
	 * Updates an object that is already stored in the database.
	 * 
	 * @param stmt  JDBC Statement for contacting the dbase
	 * @param id    per-table id of the object to be written.
	 * @param obj   an array: the first element is the BO to be stored.
	 * @exception SQLException  an error occurred during the dbase operation
	 * 
	 * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
	 * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
	 */
	public void update(Statement stmt, StorableObject obj)
		throws SQLException {
		PanelObsRun panel_run = (PanelObsRun) obj;

		StringBuffer sb = new StringBuffer();

		sb.append("grade = ");
		sb.append(panel_run.getGrade() * GRADE_CONV);
		sb.append(", rank = ");
		sb.append(panel_run.getRank());
		sb.append(", assigned_time = ");
		sb.append(panel_run.getNights() * TIME_CONV);
		sb.append(", sigma = ");
		sb.append(panel_run.getSigma() * GRADE_CONV);
		sb.append(", obs_mode = '");
		sb.append(panel_run.getObsMode());
		sb.append("'\n");

		String sqlText =
			"UPDATE "
				+ Phase1SelectStmt.runsTable.get()
				+ " SET "
				+ sb
				+ " WHERE id = "
				+ Config.getCfg().uniqueToTableId(panel_run.getId());

		stdlog_.debug(sqlText);
		stmt.executeUpdate(sqlText);

		sqlText =
			"UPDATE "
				+ Phase1SelectStmt.runsTable.get()
				+ " SET "
				+ Phase1SelectStmt.runsTable.get()
				+ ".rank = "
				+ panel_run.getRank()
				+ " WHERE "
				+ Phase1SelectStmt.runsTable.get()
				+ ".programme_id = "
				+ panel_run.getProgrammeDbId()
				+ " AND "
				+ Phase1SelectStmt.runsTable.get()
				+ ".grade = "
				+ panel_run.getGrade()
				+ " AND "
				+ Phase1SelectStmt.runsTable.get()
				+ ".tel_code = '"
				+ panel_run.getTelescope()
				+ "'\n";

		stdlog_.debug(sqlText);
		stmt.executeUpdate(sqlText);
	}

	/**
	 * Deletes an object from the database.
	 * 
	 * @param stmt  JDBC Statement for contacting the dbase
	 * @param id    per-table id of the object to be written.
	 * @exception SQLException  an error occurred during the dbase operation
	 */
	public void delete(Statement stmt, long id) throws SQLException {
	}

}
