package org.eso.ohs.persistence.dbase.phase1;

import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.SchedRun;
import org.eso.ohs.dfs.StorableObject;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;
import org.eso.ohs.persistence.dbase.DbaseIOHandler;

/**
 * I/O class for SchedRun stored in the database.
 * 
 * <p>
 * $Id: DbaseHandlerSchedRun.java,v 1.3 2005/09/05 15:09:08 ddorigo Exp $<br>
 * 
 * @author D. Dorigo
 * 
 * 
 */

public class DbaseHandlerSchedRun extends DbaseIOHandler {

    private static final String rcsid = "$Id: DbaseHandlerSchedRun.java,v 1.3 2005/09/05 15:09:08 ddorigo Exp $";

    public static final float GRADE_CONV = 10;

    public static final float TIME_CONV = 10;

    private static Logger stdlog_ = Logger
            .getLogger(DbaseHandlerSchedRun.class);

    /**
     * Constructor for CS I/O class.
     */
    public DbaseHandlerSchedRun() {
    }

    /**
     * Managed class
     * 
     * @return PanelSchedRun class
     */

    public Class getManagedClass() {
        return SchedRun.class;
    }

    /**
     * Reads the object from the database, partially reassembling it. References
     * to other business objects are stored in other fields of the array
     * 
     * @param stmt
     *            JDBC Statement for contacting the dbase
     * @param id
     *            per-table id of the object to be retrieved.
     * @return array in the format required by reassembleReferences()
     * 
     * @exception SQLException
     *                an error occurred during the dbase operation
     * @exception ObjectNotFoundException
     *                the requested object was not found in the database
     * @exception IOException
     *                if an I/O error occurred while reading the object
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public UnpackedStorableObject read(Statement stmt, long id)
            throws ObjectNotFoundException, IOException, SQLException {
        ResultSet rs = null;
        SchedRun panel_run = (SchedRun) newInstance();
        UnpackedStorableObject uso = new UnpackedStorableObject(panel_run);

        String run_sql = getRunSql();
        String sql = run_sql + "WHERE " + Phase1SelectStmt.schedRepTable.get()
                + ".id = " + id + "\n";

        stdlog_.debug(sql);
        rs = stmt.executeQuery(sql);

        if (!rs.next()) { // Only one with this id !!
            throw new ObjectNotFoundException("Object not found in database");
        }

        panel_run.setId(Config.getCfg().tableToUniqueId(id, SchedRun.class));
        setRunFields(panel_run, rs);

        rs.close();

        return uso;
    }

    private String getRunSql() {
        StringBuffer run_sql = new StringBuffer();
        run_sql
                .append("SELECT DISTINCT ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".period, ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".progid, ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".obs_mode, ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".tel, ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".from_date, ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".to_date, ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".nights, ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".pi_coi, ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".affil, ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".inst, ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".remarks, ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".title, ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".observer, ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".id, ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".run_id, ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".night_assistant, ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".support_astronomer, ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".month, ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".moon, ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".seeing, ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".rank, ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".night_fract , ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".rank_class, ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".last_mod , ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".published, ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".ipversion, ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(".observer_email \n")
                .append("FROM ")
                .append(Phase1SelectStmt.schedRepTable.get())
                .append(" ");
        return run_sql.toString();
    }

    /**
     * Read data
     * 
     * @param sql
     *            statement
     * @param whereClause
     *            where condition for retreiving the object
     * @exception SQLException
     *                an error occurred during the dbase operation
     * @exception ObjectNotFoundException
     *                the requested object was not found in the database
     * @exception ObjectIOException
     *                IO Eror occured while reading from the database
     */

    public UnpackedStorableObject[] read(Statement stmt, String whereClause,
            String otherTables)
            throws ObjectIOException, ObjectNotFoundException, SQLException {

        if (otherTables == null) {
            otherTables = "";
        }
        UnpackedStorableObject[] prog_array = null;
        SchedRun prog = null;
        String sql = getRunSql() + otherTables + " WHERE " + whereClause;
        stdlog_.debug(sql);
        ResultSet rs = stmt.executeQuery(sql);

        try {
            Vector prog_vector = new Vector();

            while (rs.next()) {

                prog = (SchedRun) newInstance();
                // New element
                prog_vector.addElement(prog);

                // Set only when attaching the object as a new
                // element
                prog.setId(Config.getCfg().tableToUniqueId(rs.getInt("id"),
                        SchedRun.class));
                setRunFields(prog, rs);
            }
            rs.close();

            prog_array = new UnpackedStorableObject[prog_vector.size()];
            for (int i = 0; i < prog_vector.size(); i++) {
                prog = (SchedRun) prog_vector.elementAt(i);
                prog_array[i] = new UnpackedStorableObject(prog);
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }

        return prog_array;
    }

    /**
     * Set the prog fields from the result set. Only for avoiding duplicated
     * code
     * 
     */
    protected void setRunFields(SchedRun schedRun, ResultSet rs)
            throws SQLException {
        schedRun.setTableId(rs.getInt("id"));
        schedRun.setPeriod(rs.getInt("period"));
        schedRun.setProgId(rs.getString("progid"));
        schedRun.setObsMode(rs.getString("obs_mode"));
        schedRun.setTelescope(rs.getString("tel"));
        schedRun.setFromDate(rs.getLong("from_date"));
        schedRun.setToDate(rs.getLong("to_date"));
        schedRun.setNight(rs.getFloat("nights"));
        schedRun.setPiCoi(rs.getString("pi_coi"));
        schedRun.setAffil(rs.getString("affil"));
        schedRun.setInstrument(rs.getString("inst"));
        schedRun.setRemarks(rs.getString("remarks"));
        schedRun.setTitle(rs.getString("title"));
        schedRun.setObserver(TextUtils.getEmptySqlStringIfNull(rs
                .getString("observer")));
        schedRun.setObserverEmail(TextUtils.getEmptySqlStringIfNull(rs
                .getString("observer_email")));
        schedRun.setRunId(rs.getLong("run_id"));
        schedRun.setNightAssistant(rs.getString("night_assistant"));
        schedRun.setSupportAssistant(rs.getString("support_astronomer"));
        String month = rs.getString("month");
        if (month == null || month.trim().equals("")) {
            month = SchedRun.ANY_MONTH;
        }
        schedRun.setMonth(month);
        schedRun.setMoon(rs.getString("moon"));
        schedRun.setSeeing(rs.getInt("seeing"));
        schedRun.setRank(rs.getInt("rank"));
        schedRun.setNightFract_(rs.getString("night_fract"));
        schedRun.setRankClass(TextUtils.getEmptySqlStringIfNull(
                rs.getString("rank_class")).trim());
        schedRun.setLastMod(rs.getString("last_mod"));
        // schedRun.setPublished(rs.getBoolean ("published") );
        schedRun.setPublished(rs.getInt("published"));
        schedRun.setIpversion(rs.getBigDecimal("ipversion"));
    }

    /**
     * Writes the object to the database, for the first time.
     * 
     * @param stmt
     *            JDBC Statement for contacting the dbase
     * @param id
     *            per-table id of the object to be written.
     * @param obj
     *            an array: the first element is the BO to be stored.
     * @exception SQLException
     *                an error occurred during the dbase operation
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void write(Statement stmt, long id, UnpackedStorableObject obj)
            throws ObjectIOException {
        stdlog_.debug("the obj must be saved with id " + id);
        SchedRun sched = (SchedRun) obj.getObject();
        long schedId = Config.getCfg().uniqueToTableId(sched.getId());
        String sqlText = buildWriteStmt(sched, schedId);
        stdlog_.debug(sqlText);

        try {
            stmt.executeUpdate(sqlText);
        } catch (SQLException e) {
            stdlog_.error("INSERT error", e);
            throw new ObjectIOException("Error executing write: " + sqlText);
        }

    }

    public String buildWriteStmt(SchedRun sched, long schedId) {

        String sqlText = "INSERT INTO " + Phase1SelectStmt.schedRepTable.get()
                + " (id, period, progid, obs_mode, tel, "
                + "from_date, to_date, nights, pi_coi, "
                + "affil, inst, remarks, title, observer, "
                + "observer_email, run_id, night_assistant, "
                + "support_astronomer, rank_class, month, moon, "
                + "seeing, rank, published, ipversion) VALUES ("
                // + sched.getId()+", "
                + schedId + ", " + sched.getPeriod() + ", " + "\'"
                + TextUtils.quoteSqlString(sched.getProgId()) + "\'" + ", "
                + "\'" + TextUtils.quoteSqlString(sched.getObsMode()) + "\'"
                + ", " + "\'" + TextUtils.quoteSqlString(sched.getTelescope())
                + "\'" + ", " + sched.getFromDate() + ", " + sched.getToDate()
                + ", " + sched.getNight() + ", " + "\'"
                + TextUtils.quoteSqlString(sched.getPiCoi()) + "\'" + ",  "
                + "\'" + TextUtils.quoteSqlString(sched.getAffil()) + "\'"
                + ", " + "\'" + TextUtils.quoteSqlString(sched.getInstrument())
                + "\'" + ", " + "\'"
                + TextUtils.quoteSqlString(sched.getRemarks()) + "\'" + ", "
                + "\'" + TextUtils.quoteSqlString(sched.getTitle()) + "\'"
                + ", " + "\'" + TextUtils.quoteSqlString(sched.getObserver())
                + "\'" + ", " + "\'"
                + TextUtils.quoteSqlString(sched.getObserverEmail()) + "\'"
                + ", " + sched.getRunId() + ", " + "\'"
                + TextUtils.quoteSqlString(sched.getNightAssistant()) + "\'"
                + ", " + "\'"
                + TextUtils.quoteSqlString(sched.getSupportAssistant()) + "\'"
                + ", " + "\'" + TextUtils.quoteSqlString(sched.getRankClass())
                + "\'" + ", " + "\'"
                + TextUtils.quoteSqlString(sched.getMonth()) + "\'" + ", "
                + "\'" + TextUtils.quoteSqlString(sched.getMoon()) + "\'"
                + ", " + sched.getSeeing() + ", " + sched.getRank() + ", "
                + sched.getPublished() + ", " + convertIpVersion(sched) + ")";
        return sqlText;
    }

    /**
     * @param sched
     * @return String never null
     */
    private String convertIpVersion(SchedRun sched) {
        String ipVersion = "null";
        if (sched.getIpversion() != null) {
            ipVersion = sched.getIpversion().toString();
        }
        return ipVersion;
    }

    /**
     * Updates an object that is already stored in the database.
     * 
     * @param stmt
     *            JDBC Statement for contacting the dbase
     * @param id
     *            per-table id of the object to be written.
     * @param obj
     *            an array: the first element is the BO to be stored.
     * @exception SQLException
     *                an error occurred during the dbase operation
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void update(Statement stmt, StorableObject obj) throws SQLException {
        SchedRun run = (SchedRun) obj;

        StringBuffer sb = new StringBuffer();

        sb.append(" period = ");
        sb.append(run.getPeriod());
        sb.append("\n");

        sb.append(", progid = '");
        sb.append(TextUtils.quoteSqlString(run.getProgId()));
        sb.append("'\n");

        sb.append(", obs_mode = '");
        sb.append(TextUtils.quoteSqlString(run.getObsMode()));
        sb.append("'\n");

        sb.append(", tel = '");
        sb.append(TextUtils.quoteSqlString(run.getTelescope()));
        sb.append("'\n");

        sb.append(", from_date = ");
        sb.append(run.getFromDate());
        sb.append("\n");

        sb.append(", to_date = ");
        sb.append(run.getToDate());
        sb.append("\n");

        sb.append(", nights = ");
        sb.append(run.getNight());
        sb.append("\n");

        sb.append(", pi_coi = '");
        sb.append(TextUtils.quoteSqlString(run.getPiCoi()));
        sb.append("'\n");

        sb.append(", affil = '");
        sb.append(TextUtils.quoteSqlString(run.getAffil()));
        sb.append("'\n");

        sb.append(", inst = '");
        sb.append(TextUtils.quoteSqlString(run.getInstrument()));
        sb.append("'\n");

        sb.append(", remarks = '");
        sb.append(TextUtils.quoteSqlString(run.getRemarks()));
        sb.append("'\n");

        sb.append(", title = '");
        sb.append(TextUtils.quoteSqlString(run.getTitle()));
        sb.append("'\n");

        sb.append(", observer = '");
        sb.append(TextUtils.quoteSqlString(run.getObserver()));
        sb.append("'\n");

        sb.append(", run_id = ");
        sb.append(run.getRunId());
        sb.append("\n");

        sb.append(", month = '");
        sb.append(run.getMonth());
        sb.append("'\n");

        sb.append(", moon = '");
        sb.append(run.getMoon());
        sb.append("'\n");

        sb.append(", seeing = ");
        sb.append(run.getSeeing());
        sb.append("\n");

        sb.append(", rank = ");
        sb.append(run.getRank());
        sb.append("\n");

        sb.append(", rank_class = '");
        sb.append(TextUtils.quoteSqlString(run.getRankClass()));
        sb.append("'\n");

        sb.append(", published = ");
        sb.append(run.getPublished());
        sb.append("\n");

        sb.append(", ipversion = ");
        sb.append(convertIpVersion(run));
        sb.append("\n");

        sb.append(", observer_email = '");
        sb.append(TextUtils.quoteSqlString(run.getObserverEmail()));
        sb.append("'\n");

        String sqlText = "UPDATE " + Phase1SelectStmt.schedRepTable.get() + ""
                + " SET " + sb + " WHERE id = " + run.getTableId();

        stdlog_.debug(sqlText);
        stmt.executeUpdate(sqlText);
    }

    /**
     * Updates an object that is already stored in the database.
     * 
     * @param stmt
     *            JDBC Statement for contacting the dbase
     * @param id
     *            per-table id of the object to be written.
     * @param obj
     *            an array: the first element is the BO to be stored.
     * @exception SQLException
     *                an error occurred during the dbase operation
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void update(Statement stmt, long id, UnpackedStorableObject objSpec)
            throws SQLException {
        stdlog_.debug("update object");
        StorableObject obj = objSpec.getObject();
        update(stmt, obj);
        stdlog_.debug("update object done");
    }

    /**
     * Deletes an object from the database.
     * 
     * @param stmt
     *            JDBC Statement for contacting the dbase
     * @param id
     *            per-table id of the object to be written.
     * @exception SQLException
     *                an error occurred during the dbase operation
     */
    public void delete(Statement stmt, StorableObject obj)
            throws ObjectIOException {
        SchedRun schedRun = (SchedRun) obj;
        long obCount = checkObsBlockConstraint(stmt, schedRun.getRunId());
        if (obCount == 0) {
            deleteSchedRun(stmt, schedRun.getTableId());
        } else {
            throw new ObjectIOException("There are " + obCount
                    + " observing blocks referring to this run.");
        }

    }

    /**
     * @param stmt
     * @param id
     * @throws ObjectIOException
     */
    private void deleteSchedRun(Statement stmt, long id)
            throws ObjectIOException {
        String sqlText = " \n DELETE  FROM "
                + Phase1SelectStmt.schedRepTable.get() + " WHERE id = " + id;
        stdlog_.debug(sqlText);
        try {
            stmt.executeUpdate(sqlText);
        } catch (SQLException e) {
            stdlog_.error("delete error", e);
            throw new ObjectIOException("Error executing delete: " + sqlText);
        }
    }

    /**
     * @param stmt
     * @param id
     * @return long number of obs blocks
     * @throws ObjectIOException
     */
    private long checkObsBlockConstraint(Statement stmt, long runId)
            throws ObjectIOException {

        ResultSet rs = null;
        long obCount = 0;

        String sql = " \n SELECT count(*)  FROM "
                + Phase1SelectStmt.obsBlocksTable.get() + " WHERE run_id = "
                + runId;
        try {

            stdlog_.debug(sql);
            rs = stmt.executeQuery(sql);
            if (!rs.next()) { // Only one with this id !!
                throw new ObjectIOException("Object not found in database");
            }
            obCount = rs.getLong(1);
        } catch (SQLException e1) {
            // TODO Auto-generated catch block
            e1.printStackTrace();
        }
        try {
            if (rs != null) {
                rs.close();
            }
        } catch (SQLException e2) {
            // TODO Auto-generated catch block
            e2.printStackTrace();
        }
        return obCount;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eso.ohs.dbase.DbaseIOHandler#delete(java.sql.Statement, long)
     */
    public void delete(Statement stmt, long id)
            throws ObjectIOException, SQLException {
        // TODO Auto-generated method stub

    }

}
