package org.eso.ohs.persistence.dbase.phase2;

import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.RunProperties;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;
import org.eso.ohs.persistence.dbase.DbaseIOHandler;
import org.eso.ohs.persistence.dbase.phase1.Phase1SelectStmt;

/**
 * I/O class for Masks stored in the database.
 *
 * <p>
 * $Id: DbaseHandlerRunProperties.java,v 1.5 2005/10/10 16:20:08 ddorigo Exp $
 * <br>
 * @author Bob Kemp
 * 
 * @see org.eso.ohs.persistence.dbase.phase2.DbaseMgr
 */

public class DbaseHandlerRunProperties extends DbaseIOHandler {

    private static final String IS_CHILEAN_COL = "is_chilean";
    private static final String PRE_IMAGING_COL = "pre_imaging";
    private static final String CARRYOVER_COL = "carryover_not_derived";
    private static final String ID_COL = "id";
    private static final String TYPE_COL ="gto";
    private static final String IS_TIME_CRITICAL_COL = "time_criti";
    private static final String RUN_STARTED = "run_started";
  

    /**
     * Constructor for CS I/O class.
     */
    public DbaseHandlerRunProperties() {
    }


    /**
     * Managed class
     * @return Mask class
     */

    public Class getManagedClass() {      
	return RunProperties.class;
    }

    /**
     * Reads the object from the database, partially reassembling it.
     * References to other business objects are stored in other fields
     * of the array
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be retrieved.
     * @return	    array in the format required by reassembleReferences()
     * 
     * @exception SQLException
     *		    an error occurred during the dbase operation
     * @exception ObjectNotFoundException
     *		    the requested object was not found in the database
     * @exception IOException
     *              if an I/O error occurred while reading the object
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public UnpackedStorableObject read(Statement stmt, long id)
			throws ObjectNotFoundException,
				IOException, SQLException {
    	    throw new IOException("Operation not supported for this object");
    }

    /**
     * Read data
     * 
     * @param sql
     *            statement
     * @param whereClause
     *            where condition for retreiving the object
     * @exception SQLException
     *                an error occurred during the dbase operation
     * @exception ObjectNotFoundException
     *                the requested object was not found in the database
     * @exception ObjectIOException
     *                IO Eror occured while reading from the database
     */

    public UnpackedStorableObject[] read(Statement stmt, String whereClause,
            String otherTables)
            throws ObjectIOException, ObjectNotFoundException, SQLException {

        if (otherTables == null) {
            otherTables = "";
        }
        UnpackedStorableObject[] ob_array = null;
        RunProperties obj = null;
        String sql = buildSqlQuery(whereClause, otherTables);
        stdlog_.debug(sql);
        ResultSet rs = stmt.executeQuery(sql);

        try {
            Vector obj_vector = new Vector();

            while (rs.next()) {

                obj = new RunProperties();
                obj.setId(Config.getCfg().tableToUniqueId(rs.getLong(ID_COL), RunProperties.class));
                obj.setDbaseId(rs.getLong(ID_COL));
                    setFields(obj, rs);
                // New element
                obj_vector.addElement(obj);

               
            }
            rs.close();

            ob_array = new UnpackedStorableObject[obj_vector.size()];
            for (int i = 0; i < obj_vector.size(); i++) {
                obj = (RunProperties) obj_vector.elementAt(i);
                ob_array[i] = new UnpackedStorableObject(obj);
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }

        return ob_array;
    }

    public static String buildSqlQuery(String whereClause, String otherTables) {
        String sql = getSelectSql() + otherTables + "\n WHERE " 
        +   Phase1SelectStmt.runsTable.get()+
        ".id *= "+Phase2SelectStmt.usgFileTable.get()+".run_id " +
        "\n AND " + Phase1SelectStmt.programmesTable.get()+
        ".id  = " + Phase1SelectStmt.runsTable.get()+
        ".programme_id " +
        " \n AND " 
        +  whereClause+
        " ORDER BY "+ Phase1SelectStmt.runsTable.get()+ ".id  " ;
        return sql;
    }
    
    
    
    /**
     * Set the prog fields from the result set. Only for avoiding duplicated
     * code
     *  
     */
    protected void setFields(RunProperties obj, ResultSet rs) throws SQLException {       
        obj.setCarryOver(rs.getBoolean(CARRYOVER_COL));   
        obj.setPreImaging(rs.getBoolean(PRE_IMAGING_COL));   
        obj.setChilean(rs.getBoolean(IS_CHILEAN_COL));            
        obj.setTimeCritical(rs.getBoolean(IS_TIME_CRITICAL_COL));            
        obj.setType(rs.getInt(TYPE_COL));            
        obj.setRunStarted(rs.getBoolean(RUN_STARTED));            
           }

    /**
     * @return Returns the run_sql.
     */
    private static String getSelectSql() {
         String run_sql = 
             "SELECT DISTINCT " + 
             Phase2SelectStmt.usgFileTable.get()+ "."+CARRYOVER_COL+",  " +
             Phase2SelectStmt.usgFileTable.get()+ "."+PRE_IMAGING_COL+",  " +
             Phase2SelectStmt.usgFileTable.get()+ "."+IS_CHILEAN_COL+",  "+
             Phase2SelectStmt.usgFileTable.get()+ "."+RUN_STARTED+",  "+
             Phase2SelectStmt.usgFileTable.get()+ "."+IS_TIME_CRITICAL_COL+",  "+
             Phase1SelectStmt.runsTable.get()+ "."+ID_COL+",  "+
             Phase1SelectStmt.programmesTable.get()+ "."+TYPE_COL+" " 
	    + " FROM " + Phase2SelectStmt.usgFileTable.get()+ 
        ",  " + Phase1SelectStmt.runsTable.get()+ 
        ",  " + Phase1SelectStmt.programmesTable.get() ;
        return run_sql;
    }
    /**
     * Writes the object to the database, for the first time.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @param obj   an array: the first element is the BO to be stored.
     * @exception SQLException  an error occurred during the dbase operation
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void write(Statement stmt, long id, UnpackedStorableObject obj)
			throws SQLException {
        throw new SQLException("Operation not supported for this object");
    }


    /**
     * Updates an object that is already stored in the database.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @param obj   an array: the first element is the BO to be stored.
     * @exception SQLException  an error occurred during the dbase operation
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void update(Statement stmt, long id, UnpackedStorableObject obj)
			throws SQLException {
        throw new SQLException("Operation not supported for this object");
    }


    /**
     * Deletes an object from the database.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @exception SQLException  an error occurred during the dbase operation
     */
    public void delete(Statement stmt, long id)
			throws ObjectIOException {
	    throw new ObjectIOException("Operation not supported for this object");

    }

}


