package org.eso.ohs.phase2.actions;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;
import java.util.LinkedList;
import java.util.ListIterator;

import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.text.BadLocationException;
import javax.xml.transform.TransformerException;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.widgets.HTMLDisplayWidget;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.ReflectionException;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.OBDifferenceBuilder;
import org.eso.ohs.persistence.OBDifferenceConfigurationException;
import org.eso.ohs.persistence.OBDifferenceXSLTransform;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;


/**
 * Class OBDiffAction
 *
 * Action class to display the difference between two ObservationBlock objects
 *
 * @version $Revision: 1.5 $
 * @author pnunes, Feb 10, 2005
 */

// $Id: OBDiffAction.java,v 1.5 2005/11/22 14:36:17 pnunes Exp $ 
public class OBDiffAction extends ActionSuperclass {
    
    private static final String configDir_ =        
       "config" +  File.separator;

    private ObjectContainer objectContainer_;
    
    private static HTMLDisplayWidget htmlWidget_ = new HTMLDisplayWidget("Compare Observation Blocks", true, true, true, configDir_+"diffOB.css");
    
    /**
     * @param comp
     * @param name
     */
    public OBDiffAction(ObjectContainer objectContainer, String name) {
        super((JComponent)objectContainer,name);
        objectContainer_ = objectContainer;
        
    }

    /* (non-Javadoc)
     * @see org.eso.ohs.apps.actions.ActionSuperclass#actionPerformedImpl(java.awt.event.ActionEvent)
     */
    public void actionPerformedImpl(ActionEvent ev) {
               
        Long [] distinctIds = getDistinctSelection();
                
        // wrong selection
        if(distinctIds.length !=2) {
           JOptionPane.showMessageDialog(
                objectContainer_.getTopLevelAncestor(),
                new Object[] {"Please select two distinct Objects","",""},
                "Wrong selection",
                JOptionPane.ERROR_MESSAGE
     	    	);
     	    return;
        } else {                                                        
            try {
                Compare(distinctIds[0].longValue(), distinctIds[1].longValue());
            } catch (OBDifferenceConfigurationException e) {
                messageDialog(objectContainer_.getTopLevelAncestor(),"Error",e.toString(),"","");
            }
        }
    }
    
    private Long [] getDistinctSelection() {    	    
        Long [] obIds = objectContainer_.getAllSelected();
        LinkedList selectedIds = new LinkedList();                
        
        for (int i = 0; i < obIds.length; i++) {			
        	if ( !selectedIds.contains(obIds[i]) )
        		selectedIds.add(obIds[i]);
		}        
       
        ListIterator iter = selectedIds.listIterator();
        Long [] res = new Long[selectedIds.size()];
        int i = 0;
        while (iter.hasNext()) {
			Long id = (Long) iter.next();
			res[i] = id;
			i++;
		}
    	return(res);
    }
    
    
    
    /**
     * Compares two OBs or CBs, given a pair of IDs
     * Checks the instances
     * 
     * @param id1
     * @param id2
     * @throws OBDifferenceConfigurationException
     */
    private void Compare (long id1, long id2) throws OBDifferenceConfigurationException {
        
        Long [] obIds = objectContainer_.getAllSelected();
        
        ObservationBlock ob1, ob2;
        CalibrationBlock cb1, cb2;
        Object obj1, obj2;

        // load the two OBs and compare them    
            try {
                Media device = Config.getCfg().getMediaFromId(id1);
                Class objType = Config.getCfg().getClassFromId(id1);
                obj1  = ObjectManager.getObjectManager().getBusObj(device,id1,objType);
                
                device = Config.getCfg().getMediaFromId(id2);
                objType = Config.getCfg().getClassFromId(id2);
                obj2  = ObjectManager.getObjectManager().getBusObj(device,id2,objType);

                /* Check instances, verify OBs or CBs,  
                 * call comparison methods
                 * and show result in a HTMLWidget
                 * */
                
                String html = "";
                    
                if ( obj1 instanceof ObservationBlock ) { 
                    ob1 = (ObservationBlock)obj1; 
                    ob2 = (ObservationBlock)obj2;
                    
                    if (! ob1.verifyCheck()) {
                        messageDialog(objectContainer_.getTopLevelAncestor(),"Invalid block","1st Observation block not complete","","");
                        return;
                                                       
                        } else {
                            if (! ob2.verifyCheck() ) {
                                messageDialog(objectContainer_.getTopLevelAncestor(),"Invalid block","2nd Observation block not complete","","");
                                return;
                            }
                        }    
                        html = CompareCB(ob1,ob2);
                    }
                    else
                        if ( obj1 instanceof CalibrationBlock ) {
                            cb1 = (CalibrationBlock)obj1;
                            cb2 = (CalibrationBlock)obj2;
                            html = CompareCB(cb1,cb2);
                            
                            if (! cb1.verifyCheck()) {
                                messageDialog(objectContainer_.getTopLevelAncestor(),"Invalid block","1st Calibration block not complete","","");
                                return;
                            } else {
                                if (! cb2.verifyCheck() ) {
                                    messageDialog(objectContainer_.getTopLevelAncestor(),"Invalid block","2nd Calibration block not complete","","");
                                    return;
                                
                                }
                            }
                        }                                                       
                    htmlWidget_.setContents(html);                       
                    htmlWidget_.setVisible(true);
                    
                } catch (ReflectionException e1) {

                    e1.printStackTrace();
                } catch (IOException e1) {

                    e1.printStackTrace();
                } catch (TransformerException e1) {

                    e1.printStackTrace();
                } catch (ObjectNotFoundException e) {
 
                    e.printStackTrace();
                } catch (ObjectIOException e) {

                    e.printStackTrace();
                } catch (BadLocationException e) {

                    e.printStackTrace();
                } 
        
    }    
                        	
    /**
     * Shows a message dialog
     * 
     * @param parent
     * @param title
     * @param line1
     * @param line2
     * @param line3
     */
    private void messageDialog( Component parent, String title , String line1, String line2, String line3 ) {
        
        JOptionPane.showMessageDialog(
                parent,
     		new Object[] {
     		  line1,
     		  line2,
     		  line3
     		},
     		title,
     		JOptionPane.ERROR_MESSAGE
     	    );
    }
    

    private String CompareCB(CalibrationBlock cb1, CalibrationBlock cb2 ) throws IOException, TransformerException, IOException, ReflectionException, OBDifferenceConfigurationException {
        
        OBDifferenceBuilder obDiffBuilder = new OBDifferenceBuilder( configDir_+ "OBDiffConfig.xml");
        OBDifferenceXSLTransform xslTransform = new OBDifferenceXSLTransform( configDir_ + "OB.xsl"); 
        String xml,html;
                
        if (cb1 instanceof ObservationBlock) {
        
            xml = obDiffBuilder.compareObservationBlocksToXML((ObservationBlock)cb1,(ObservationBlock)cb2);        
            html = xslTransform.simpleTransformToString(xml);
        } else {
            xml = obDiffBuilder.compareObservationBlocksToXML(cb1,cb2);        
            html = xslTransform.simpleTransformToString(xml);
        }
        
        return(html);
        
    }

}





