package org.eso.ohs.phase2.apps.p2pp.dbb;

import org.eso.ohs.core.dbb.client.Coordinate;
import org.eso.ohs.persistence.dbase.phase2.DbaseHandlerTG;

/**
 *  A transducer to convert set/get of String values into set/get
 *  on an int property.  The target property's name is given to the
 *  constructor.
 *
 *  <p>
 *  For example, users will call it's
 *  setValue/getValue methods.  Those calls will be translated into
 *  calls to setX/getX on the object given to the constructor.
 *
 *  <p>
 * $Id: RightAscension.java,v 1.7 2005/04/11 13:53:44 tcanavan Exp $
 */

public class RightAscension extends Coordinate {

  public void checkBounds(int sign, int hours, int minutes,
			  int seconds, int milliSecs) {
// System.out.println("sign   = " + sign   );
// System.out.println("hours  = " + hours  );
// System.out.println("minutes= " + minutes);
// System.out.println("seconds= " + seconds);
// System.out.println("milliSecs   = " + milliSecs   );
// System.out.println();

    if (sign < 0 || hours >= 24 || minutes >= 60
	|| seconds >= 60 || milliSecs >= 1000) {
      throw new IllegalArgumentException("Illegal time value");
    }
  }

  /**
   * Converts data from a String to a meta-level equivalent of the
   * actual property's type, i.e. to a java.lang.Integer
   * representing an int.  Used with reflection to set
   * the new property value.
   *
   * @param value the new value of the property.
   */
  public Object toDatabaseData(String value) {
    Integer num = (Integer)super.toDatabaseData(value);

    return new Integer(num.intValue()*DbaseHandlerTG.DEGREES_PER_HOUR);
  }

  /**
   * Converts data to a String from the meta-level equivalent of the
   * actual property's type, i.e. from a java.lang.Long
   * to a long int.  Used with reflection to get
   * the current property value.
   *
   * @param value	value of the property.
   */
  public String fromDatabaseData(Object obj) {
    int n = new Integer(super.fromDatabaseData(obj)).intValue();

    return String.valueOf(n/DbaseHandlerTG.DEGREES_PER_HOUR);
  }

}

