package org.eso.ohs.phase2.apps.p2pp.views;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.eso.ohs.core.docview.gui.DocView;
import org.eso.ohs.core.gui.models.BoNameModel;
import org.eso.ohs.core.gui.models.LimitedCharsModel;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.ToolTipJList;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.ObservationDescription;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.instruments.TemplateSignature;
import org.eso.ohs.instruments.TemplateSignatureFactory;
import org.eso.ohs.persistence.Config;

/**
 * This is a specialised JPanel with subcomponents linked to the
 * given Object Descriptions object's fields.  When this component is
 * displayed, changes to it will be reflected in the Target
 * object.  Likewise changes to the Object Description  object will be
 * reflected in the displayed fields.  Some fields of an Object Dewcription
 * may not be visible.  Some fields may not be modifiable.
 * 
 * <p>
 * $Id: ODView.java,v 1.8 2004/02/17 14:40:33 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 */

public class ODView extends ObjectView implements PropertyChangeListener {

    /** ObservationDescription currently displayed, or null */
    ObservationDescription currentOD_;

    /** Add Template button */
    private JButton addTemplate_;
    
    /** Removes template button*/
    private JButton removeTemplate_ ;

    /** Duplicate template button*/
    private JButton duplicateTemplate_;
  
    /** GUI component for the Name field */
    private DocView fieldName_;

    /** GUI component for the UserComments field */
    private DocView fieldUserComments_;
    
    /** List of instrument types*/
    private JList instrumentTypes_;

    /** List for template names*/
    private ToolTipJList templateNames_;

    /** Instrument that this od refers to*/
    private Instrument inst_;

    /** Table for TemplateSignature array in od*/
    private ODTemplateTable templateTable_;

    /** Data model for TemplateSignature*/
    private ODGridModel templateModel_;

    /** Label name*/
    private JLabel nameLabel_;


    /**
     * Construct the JPanel and add the subcomponents and listeners, etc.
     * Some fields of an OD may not be visible.
     * Some fields may not be modifiable.
     * 
     * <p> Initially, no OD is displayed.
     */
    public ODView() {
	JLabel lab;
	JTextField tf;
	JTextArea texta;
	JPanel panel = new JPanel();

	panel.setLayout(new GridBagLayout());
	GridBagConstraints gbc = new GridBagConstraints();
	setLayout(new GridBagLayout());

	gbc.fill       = GridBagConstraints.HORIZONTAL;
	gbc.gridx      = 0;
	gbc.gridy      = 1;
	gbc.weightx    = 1;
	gbc.weighty    = 1;
	gbc.gridwidth  = 1;
	gbc.gridheight = 1;  
	nameLabel_ = new JLabel("Name: ");
	nameLabel_.setHorizontalAlignment(JLabel.RIGHT);
	panel.add(nameLabel_,gbc);

	gbc.weightx=10;
	gbc.gridx      = 2;
	gbc.gridwidth  = 1;
	tf = new JTextField(Config.getCfg().getMaxNameLength());   
	tf.setDocument(new BoNameModel());
	fieldName_ = new DocView(tf);
	panel.add(fieldName_, gbc);

	/* next row of grid */
	gbc.weightx = 1;
	gbc.gridx = 0;
	gbc.gridy++;      
	gbc.gridwidth  = 2;
	lab = new JLabel("User Comments: ");
	lab.setHorizontalAlignment(JLabel.RIGHT);
	panel.add(lab, gbc);

	gbc.fill       = GridBagConstraints.BOTH;
	gbc.weighty    = 100;
	gbc.gridx      = 2;
	gbc.gridwidth=2;      
	texta = new JTextArea();
	texta.setDocument(new LimitedCharsModel());
	fieldUserComments_ = new DocView(texta);
	panel.add(new JScrollPane(fieldUserComments_), gbc);

	panel.setBorder(BorderFactory.createEtchedBorder());
	gbc.fill       = GridBagConstraints.BOTH;
	gbc.gridwidth  = 2;
	gbc.gridheight = 1;
	gbc.weightx    = 70;
	gbc.weighty    = 1;
	gbc.gridy      = 1;
	gbc.gridx      = 1;
	add(panel,gbc);



	// 3 &  4 column of grid which containts the 
	// Catagory and Template lists
	//gbc.insets = new Insets(3,3,4,1);

	gbc.fill       = GridBagConstraints.HORIZONTAL;
	gbc.gridwidth  = 1;
	gbc.gridheight = 1;
	gbc.weightx    = 0;
	gbc.weighty    = 0;
	gbc.gridx=3;
	gbc.gridy=0;
	lab = new JLabel("Template Type");
	add(lab,gbc);

	gbc.gridx++;
	lab = new JLabel("Template");
	add(lab);       

	gbc.fill       = GridBagConstraints.VERTICAL;
	gbc.gridwidth  = 1;
	gbc.gridheight = 1;
	gbc.weightx    = 0;
	gbc.weighty    = 0;

	gbc.gridy      = 1;
	gbc.gridx      = 3;
	instrumentTypes_ = new JList();
	instrumentTypes_.setPrototypeCellValue("Template Type"); 
	final JScrollPane typeScrollPane = new JScrollPane(
		instrumentTypes_	
	    );
	add(instrumentTypes_,gbc);
	gbc.weightx=100;
	gbc.weighty=0;
	gbc.gridx++;

	gbc.fill       = GridBagConstraints.BOTH;
	templateNames_ = new ToolTipJList();      
	templateNames_.setPrototypeCellValue("mmmmMmmmmMmmmmMeeeeeeee");
	final JScrollPane nameScrollPane = new JScrollPane(
		templateNames_
	
	    );
	add(nameScrollPane, gbc);
	//add(templateNames_,gbc);

	gbc.fill       = GridBagConstraints.HORIZONTAL;
	gbc.gridwidth  = 1;
	gbc.gridheight = 1;
	gbc.weightx    = 10;
	gbc.weighty    = 1;

	gbc.gridy      = 1;
	gbc.gridx      = 5;            
	initButtons();
	Box buttonPanel = Box.createVerticalBox();

	buttonPanel.add(addTemplate_);
	buttonPanel.add(removeTemplate_,gbc);
	buttonPanel.add(duplicateTemplate_,gbc);
	buttonPanel.add(Box.createVerticalGlue());
	add(buttonPanel,gbc);
	gbc.fill       = GridBagConstraints.BOTH;
	gbc.gridwidth  = 5;
	gbc.gridheight = 1;
	gbc.weightx    = 1;
	gbc.weighty    = 1000;

	gbc.gridy      = 3;
	gbc.gridx      = 1;

	gbc.insets = new Insets(5, 3, 4, 1);
	templateTable_ = new ODTemplateTable();
	add(new JScrollPane(templateTable_),gbc);
	addListListeners();
	addActionListeners();
	setObject(null);                  
    }
      
    /** Initialise instrument and template lists*/


    private void initLists() {
    inst_ = InstrumentList.getInstance().getInstrument(currentOD_.getInstrument());
	instrumentTypes_.setListData(Instrument.instrumentTypes);
		    // set a fixed prototypical size of list cell.

	instrumentTypes_.setSelectedIndex(0);    
	if(inst_ != null) {
	    templateNames_.setListData(
		inst_.getTemplateNames((String)instrumentTypes_.getSelectedValue())
	    );
	    templateNames_.setCellRenderer(new TemplateListCellRenderer(inst_));
	}
    }


    /** Initialise add and remove template buttons*/
    private void initButtons() {
	addTemplate_    = new    JButton("Add");
	removeTemplate_ = new    JButton("Delete");    
	duplicateTemplate_ = new    JButton("Duplicate");        
    }



    /**
     * Add List Selection listeners for the instrumet type and 
     * template name JLists.
     */
    private void addListListeners () {           
	ListSelectionListener lsl = new ListSelectionListener() {
	    public void valueChanged(ListSelectionEvent ev) {		  
		if (ev.getValueIsAdjusting()) {
		    return;
		}

		String type = (String)instrumentTypes_.getSelectedValue();                                         
		int index;
		if (ev.getSource() == instrumentTypes_) {	    
		   type = (String)instrumentTypes_.getSelectedValue();
		   addTemplate_.setEnabled(false);
		}
		else if (ev.getSource() == templateNames_) {
		    index = templateNames_.getSelectedIndex();
		    if(index >= 0)
			addTemplate_.setEnabled(true);			
		    else 
			addTemplate_.setEnabled(false);                       
		}	
		if(type != null && ev.getSource() != templateNames_) {
if (inst_ == null) return;  // XXX
		    templateNames_.setListData(inst_.getTemplateNames(type));
		}	
	   }
       };     
       instrumentTypes_.addListSelectionListener(lsl);     
       templateNames_.addListSelectionListener(lsl);
       templateNames_.addMouseListener( new MouseAdapter() {
	   public void mouseClicked(MouseEvent evt) {                     
	       if(evt.getClickCount()==2)
		   addTemplate();
	   }
       });   
    }


    /** Add a template to the observation description*/
    private void addTemplate() {
	try {                  
	    Object[] templateNames = templateNames_.getSelectedValues();
	    for(int i=0;i<templateNames.length;i++) {
		TemplateSignature ts = TemplateSignatureFactory.
        getTemplate(inst_,(String)templateNames[i]);                      
		int column = 0;                                     
		if(templateModel_.getColumnCount() != 0) {
		    templateTable_.stopCellEditing();
		    column = templateTable_.getSelectedColumn();
		}                                                 
		templateModel_.insertColumn(column,ts);           
	    }                                                    
	    currentOD_.setSignatures(templateModel_.getTemplateSignatures());              
	 }	   
	 catch(IOException e) {
	    ErrorMessages.announceIOError(templateTable_,e);	
	 }
    }


 
    /**
     * Add action listeners for each of the buttons in the dialog.
     */
    private void addActionListeners() {    
	addTemplate_.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent evt) {
	    addTemplate();                         
	    duplicateTemplate_.setEnabled(true);
	    removeTemplate_.setEnabled(true);
	    templateTable_.setRowColumn(0,templateModel_.getCurrColumn()); 
	    }
	});

	duplicateTemplate_.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent evt) {                                    	       
		if(templateModel_.getColumnCount() != 0) {
		    templateTable_.stopCellEditing(); 
		    int column = templateTable_.getSelectedColumn(); 
		    Object item = templateModel_.getDataItem(column);                    
		    TemplateSignature ts=null;                                          
		    if(item instanceof TemplateSignature) {
			ts = new TemplateSignature((TemplateSignature)item);
			templateModel_.insertColumn(column,ts);                       
			currentOD_.setSignatures(templateModel_.getTemplateSignatures());
			templateTable_.setRowColumn(0,templateModel_.getCurrColumn());
		   }
		}
	    }	           
	     
	});

	removeTemplate_.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent evt) {
	       templateTable_.stopCellEditing();
	       int column = templateTable_.getSelectedColumn();
	       TemplateSignature[] array;
	       templateModel_.removeColumn(column);                
	       if(templateModel_.getColumnCount() > 0) {
		   array = templateModel_.getTemplateSignatures();                     	     
	       }
	       else {
		   array = new TemplateSignature[0];
		   removeTemplate_.setEnabled(false);
		   duplicateTemplate_.setEnabled(false);
	       }
	       currentOD_.setSignatures(array);
	       templateTable_.setRowColumn(0,templateModel_.getCurrColumn());
	    } 
	});   

    }

    /**
     * Returns a reference to the currently displayed OD.
     * 
     * @return reference to an ObservationDescription.
     */
    public BusinessObject getObject() {
	return currentOD_;
    }


    /**
     * Handle a change in the property value for the associated
     * BusinessObject.  Enable or disable changes to the fields,
     * according to whether they are allowed to be modified.
     * 
     * @param ev    Property Change Event
     */
    public void propertyChange(PropertyChangeEvent ev) {
	setEnabled(isAllowedToModify(currentOD_));
    }

    /**
     * Enable or disable the fields of this panel and any
     * nested panels.  The "locked" icon is displayed
     * next to the Name field if the fields are locked.
     * 
     * @param enable	True if the fields are to be enabled,
     *			false otherwise.
     */
    public void setEnabled(boolean enable) {
	super.setEnabled(enable);

	nameLabel_.setIcon(enable ? null : locked_);
       
	fieldName_.setEnabled(enable);
	fieldUserComments_.setEnabled(enable);        
	templateNames_.setEnabled(enable);
   
    }


    /**
     * Sets a new value of the current OD.
     * This method is included to satisfy the ObjectView
     * interface, which in turn allows for generalisation of
     * other parts of the system.
     * 
     * @param newValue reference to a BusinessObject.
     */
    public void setObject(BusinessObject newOD) {
	setObject((ObservationDescription)newOD);
    }

    /**
     * Updates the DocView components to refer to the correct OD.
     * 
     * @param newOD	    ObservationDescription object to be displayed.
     */
    public void setObject(ObservationDescription newOD) {
	if (newOD == currentOD_) {
	    if(!Config.getCfg().isInEngineeringMode())	 
		return;		// nothing to do -- just return
	    if(currentOD_ != null)
		initLists();
	}

	if (currentOD_ != null) {
	    currentOD_.removePropertyChangeListener(this);
	}

	currentOD_ = newOD;

	boolean canModify = isAllowedToModify(currentOD_);

	fieldName_.setObjectProperty(
	    newOD, "Name"
	);

	fieldUserComments_.setObjectProperty(
	    newOD,"UserComments"
	);


	if (currentOD_ != null) {
	    currentOD_.addPropertyChangeListener(this);

	    templateModel_ = ODGridModel.getCachedODModel(this, currentOD_);
	    templateTable_.setModel(templateModel_);

	    if (canModify && templateModel_.getColumnCount() > 0) {          
		removeTemplate_.setEnabled(true);
		duplicateTemplate_.setEnabled(true);  	
	    }
	    else {
		removeTemplate_.setEnabled(false);
		duplicateTemplate_.setEnabled(false);
	    }

	    initLists();
	}
	else {
	    removeTemplate_.setEnabled(false);
	    duplicateTemplate_.setEnabled(false);
	}

	fieldName_.requestFocus();
      
	setEnabled(isAllowedToModify(currentOD_));
    }

}


