package org.eso.ohs.phase2.apps.p2pp.views;

import javax.swing.table.TableModel;

import org.eso.ohs.dfs.ObservationDescription;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.TemplateSignature;

/**
 * This is the interface that the ODGridModel and TemplateModel
 * must implement. This enables the one JTable to interact with 
 * the two different models 
 */

public interface TemplateModel extends TableModel {

    
    /**
     * Return the date item for a particular column
     */
    public Object getDataItem (int column);

    /**
     * Get the currently selected column
     * @return currenyly selected column
     */

    public int getCurrColumn();

    /**
     * TemplateSignatures associated with the data model
     * @return Template Signatures
     */

    public TemplateSignature[] getTemplateSignatures();

  
    /**
     * Acquisition Template signature
     */

    public TemplateSignature getAcquisition();
   
    /**
     * Observation Description
     * @return Observation Description
     */

    public ObservationDescription getOD();

    /**
     * Save changes to business object
     */

    public void saveChanges();

    /**
     * Column is a label or not
     * @param column column
     */
    
    public boolean isLabel(int column);   

    /**
     * Instrument associated with the underlying 
     * od or ob
     */
 
    public Instrument getInstrument();

    /**
     * Remove a template Signature from the grid.
     */

    public void removeColumn(int column);

    /**
     * Move a TemplateSignature around in the grid.
     */

    public void moveColumn(int sourceColumn,int destColumn);


    public void insertColumn(int column,TemplateSignature ts);

    
}
