package org.eso.ohs.phase2.icdVcs;

import java.awt.Component;

// import org.eso.vlt.base.Ccs.*;
//import org.eso.ohs.masktracker.*;


/**
 * Clearing house for low-level configuration information.
 * 
 * <p>
 * @author cmavros 2002/02/14, 1512/P2PP
 * $Id: IcdVcsCfg.java,v 1.3 2003/07/03 08:49:49 ddorigo Exp $
 */
public abstract class IcdVcsCfg {

    /** Reference to this singleton config class */
    private static IcdVcsCfg cfg_ = null;


    /** Default constructor */
    protected IcdVcsCfg() {
    }

    /**
     * Factory method to return the singleton instance of this class
     * 
     * @return current (singleton) instance of this class
     */
    public static IcdVcsCfg getCfg() {
	return cfg_;
    }


    /**
     * Sets a new singleton instance of this class.
     * For use by an application to install application-dependent
     * subclass instance.
     * 
     * @param cfg	new (singleton) instance of this class
     */
    public static void setCfg(IcdVcsCfg cfg) {
	cfg_ = cfg;
    }

    /**
     * Returns the CCS server name by which this application is to be known.
     * 
     * @return	name of the CCS server
     */
    public abstract String getCCSServer();

    /**
     * Is the user currently in Engineering mode?
     * 
     * @return	true if in visitor mode, false otherwise
     */
    public abstract boolean isInEngineeringMode();

    /**
     * Is the user currently in Visitor mode?
     * 
     * @return	true if in visitor mode, false otherwise
     */
    public abstract boolean isInVisitorMode();

    /**
     * Has the user selected cloning?
     * 
     * @return	true if in clone mode, false otherwise
     */
    public abstract boolean isInCloneMode();

    /**
     * Type of object that is currently displayed.
     * 
     * @return	Type of object to send to BOB
     */
    public abstract Class getObjectClassToSend();

    /**
     * Is there a selected object of the specified type?
     * 
     * @param objType	Type of object to be checked
     * 
     * @return	true if an object of this type has been selected
     */
    public abstract boolean hasValidSelection();

    /**
     * The id of the currently selected object of the given type.
     * 
     * @return	id of selected object, or 0 if nothing selected
     */
    public abstract long getSelection(Class objType);


    /**
     * The id of a set of selected object of the given type.
     * @param num the number of selected objects to return
     * 
     * @return	a set of Id for the selected object, or null is returned
     *          if nothing is selected
     */
    public abstract long[] getSelections(int maxNumOBs);

    /**
     * Type of object that is currently displayed.
     * 
     * @return	Type of object to send to BOB
     */
    public abstract Class getObjType(long id);


    /**
     * A Java Component on which dialog boxes can be based.
     * 
     * @return	An AWT component
     */
    public abstract Component getContainer();
  

}


