package org.eso.ohs.phase2.orang;

import java.io.File;
import java.io.FilenameFilter;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.TextUtils;

public class OrangScriptFinder {
	
	
	/**
	 * Logging class.
	 */
	protected static Logger stdlog_ =
		Logger.getLogger(OrangScriptFinder.class);
	
	
	private FilenameFilter orangFileFilter_ = new FilenameFilter() {
		public boolean accept(File dir, String name) {
			
			String fileExtensions[] = OrangScriptingBox.getSupportedScriptExtensionsImpl();
			File f = new File(dir.getAbsolutePath(), name);
			boolean rc = name.startsWith(OrangScriptingBox.orangScriptPrefix)
			&& f.isFile();
			if (rc) {
				int i = 0;
				while (i < fileExtensions.length) {
					if (name.endsWith("."+fileExtensions[i])) {
						break;
					}
					i++;
				}
				rc = (i != fileExtensions.length);
			}
			return rc;
		}
	};
	
	
	private File [] findScripts(String scriptsDirPath) throws OrangScriptException {
        
        if (scriptsDirPath == null)
            throw new OrangScriptException("Orang scripts directory not initialized");
        
		File [] scripts = new File[0];
		
        
		File scriptsDir = new File(scriptsDirPath);
		stdlog_.debug("ORANG: Searching for scripts in: "+scriptsDir.toString());
		
		
		if (scriptsDir.isDirectory()) {
			String[] files = TextUtils.sort(scriptsDir.list(orangFileFilter_));
			
			
			scripts = new File[files.length];
			for (int i = 0; i < files.length; i++) {				
				scripts[i] = new File(scriptsDir.getAbsolutePath()
						+File.separator+files[i]);
			}
		}	else {
			stdlog_.debug("ORANG: Could not find scripts.");
		}
		return scripts;
	}
	
	public String [] findNames(String scriptsDirPath) throws OrangScriptException {
		
		File [] files = findScripts(scriptsDirPath);
		
		String [] res = new String[files.length];
		
		for (int i = 0; i < files.length; i++) {
			String name = removePrefixExt(files[i].getName());
			res[i] = name;
		}
		
		return(res);
		
	}
	
	
	public File findScript(String algorithmName , String scriptsDirPath) throws OrangScriptException {
		
		File [] availableScripts = findScripts(scriptsDirPath);
		
		for (int i = 0; i < availableScripts.length; i++) {
			File file = availableScripts[i];
			
			String name = file.getName();
			
			if (algorithmName.equals(removePrefixExt(name)))
				return(file);
		}
		
		return(null);
	}
	
	private String removePrefixExt(String name) {
		
		String prefix = OrangScriptingBox.orangScriptPrefix;		
		String fileExtensions[] = OrangScriptingBox.getSupportedScriptExtensionsImpl();
		
		String res = name.substring(prefix.length());
		
		for (int i = 0; i < fileExtensions.length; i++) {
			String ext = fileExtensions[i];
			if ( res.endsWith(ext))
				res = res.substring(0,res.length()-ext.length()-1);
		}
		
		return (res);
		
		
	}
	
	
	
	
}
