/*
 * Created by amchavan on Aug 30, 2005
 */
package org.eso.ohs.phase2.timeline;

import java.util.Date;

import org.eso.ohs.phase2.visibility.PCF;
import org.eso.ohs.phase2.visibility.PCFInterval;


/**
 * A Timeline implementing the "early-greedy" algorithm.
 * 
 * Tasks are scheduled in order of priority, highest priority first.
 * Each task is scheduled at the earliest possible "best" time for
 * that task, as inferred by the task's preference PCF 
 * (returned by the
 * {@link org.eso.ohs.phase2.timeline.TimelineTask#getPreference()} 
 * method).
 * 
 * The highest priority task is scheduled anywhere on the timeline.
 * The second best can go anywhere but for the slot allocated to the first
 * task, etc.
 * 
 * @version $Revision: 1.5 $
 * @author amchavan, Aug 30, 2005
 */

// $Id: EarlyGreedy.java,v 1.5 2005/11/09 17:28:09 amchavan Exp $

public class EarlyGreedy extends Timeline {

    /** Basic constructor */
    public EarlyGreedy() {
        super();
    }

    /**
     * Constructor
     * 
     * @param tasks  The list of tasks to put on the timeline
     * @param start  The starting datetime for the timeline
     * @param end    The closing datetime for the timeline
     */
    public EarlyGreedy( TimelineTask[] tasks, Date start, Date end ) {
        super( tasks, start, end );
    }

    /**
     * Implements the "early greedy" algorithm.
     * @see org.eso.ohs.phase2.timeline.Timeline#compute()
     */
    public void compute() {
        // For each task (in descending order of priority)
        for( int i = 0; i < tasks.length; i++ ) {
            
            // Retrieve the new candidate task
            TimelineTask task = tasks[i];
            
            // Retrieve the candidate's preference PCF and remove 
            // from it all unsuitable intervals (that is, intervals
            // already taken up by previously scheduled tasks)
            PCF pref = removeUnsuitableIntervals( task );
            
            // If no suitable intervals are left, this task cannot be scheduled
            PCFInterval earlymin = pref.getEarliestMin();
            if( earlymin.getValue() == NOGO )
                continue;
            
            // OK! Found the earliest best start time for the task,
            //     let's place it on the timeline right there
            addTask( task, earlymin.getStart() );
        }
    }
}
